#include "StdAfx.h"
#include "Tour.h"
#include <algorithm>
#include <functional>


Tour::Tour(void)
{
}


Tour::~Tour(void)
{
}


Tour::Tour( CoordMatrix* mat )
{


}


Tour::Tour( CoordMatrix* mat, std::vector< int > tour )
{


}

// Do standard 2-opt operation between selected cities
void Tour::DoTwoOpt( const int& c1, 
					 const int& c2,
					 const int& c3,
					 const int& c4 )
{
	// Feasible exchanges only
	if ( c3 == c1 || c3 == c2 || c4 == c1 || c4 == c2 ) return;	
	
	// Leave values at positions c1, c4
	// Swap c2, c3 values
	// c3 -> c2
	// c2 -> c3
	int tmp = cities.at( c2 );
	cities[ c2 ] = cities.at( c3 );
	cities[ c3 ] = tmp;	
}


// Do standard 3-opt operation between selected cities 

void Tour::DoThreeOpt( const int& c1, 
					   const int& c2,
					   const int& c3,
					   const int& c4,
					   const int& c5,
					   const int& c6 )

{
	// Store original orderings
	int c1old = cities.at( c1 );
	int c2old = cities.at( c2 );
	int c3old = cities.at( c3 );
	int c4old = cities.at( c4 );
	int c5old = cities.at( c5 );
	int c6old = cities.at( c6 );

	double old_distance = TourDistance();	
		

	// Swap 1: 40526
	// c2 -> c4	
	// c4 -> c6
	// c6 -> c2
	int tmp1 = cities.at( c2 );	
	cities[ c2 ] = cities.at( c4 );	
	cities[ c4 ] = cities.at( c6 );
	cities[ c6 ] = tmp1;

	double new_distance1 = TourDistance();

	// Re-instate if not an immprovement
	if ( new_distance1 > old_distance )
	{
		cities[ c1 ] = c1old;
		cities[ c2 ] = c2old;
		cities[ c3 ] = c3old;
		cities[ c4 ] = c4old;
		cities[ c5 ] = c5old;
		cities[ c6 ] = c6old;
	}
	else
	{
		c1old = cities.at( c1 );
		c2old = cities.at( c2 );
		c3old = cities.at( c3 );
		c4old = cities.at( c4 );
		c5old = cities.at( c5 );
		c6old = cities.at( c6 );

		old_distance = new_distance1;
	}


	// Swap 2: 
	// c2 -> c4	
	// c4 -> c5
	// c5 -> c2
	tmp1 = cities.at( c2 );	
	cities[ c2 ] = cities.at( c4 );	
	cities[ c4 ] = cities.at( c5 );		
	cities[ c5 ] = tmp1;

	new_distance1 = TourDistance();

	// Re-instate if not an immprovement
	if ( new_distance1 > old_distance )
	{
		cities[ c1 ] = c1old;
		cities[ c2 ] = c2old;
		cities[ c3 ] = c3old;
		cities[ c4 ] = c4old;
		cities[ c5 ] = c5old;
		cities[ c6 ] = c6old;
	}	
}


// Get total distance of tour
double Tour::TourDistance() const
{
	double dist = 0.0;

	int size = (int) cities.size();
	int c1, c2;

	for ( int i = 0; i < size - 1; i++ )
	{
		c1 = cities.at( i );
		c2 = cities.at( i + 1 );
		dist += Distance( c1, c2 );
	} 

	// And back to the beginning city
	c1 = cities.at( size - 1 );
	c2 = cities.at( 0 );
	dist += Distance( c1, c2 );

	return dist;
}


// Generate arbitrary tour
void Tour::CreateRandomTour()
{
	Reset();

	for ( int i = 0; i < matrix->size(); i++ )
	{
		cities.push_back( i );
	}

	random_shuffle( cities.begin() + 1, cities.end() );
}

// Generate arbitrary tour
void Tour::CreateTour()
{
	Reset();

	for ( int i = 0; i < matrix->size(); i++ )
	{
		cities.push_back( i );
	}
}

// Get distance bewteen selected cities
double Tour::Distance( const int& c1, 
		               const int& c2 ) const
{
	return matrix->Distance( c1, c2 );
}

// Set pointer to cost / distance matrix object
void Tour::SetMatrix( CoordMatrix* mat )
{
	matrix = mat;
}

// Reset existing tour data
void Tour::Reset()
{
	std::vector< int >().swap( cities );
}

// Return the number of cities in the tour
int Tour::TourSize() const
{
	return (int) cities.size();
}

// Get the tour city, given the index passed to it
int Tour::GetCity( const int& index )
{
	int node = cities.at( index );
	return node;
}

// Get tour from the set of nearest neighbours
void Tour::CreateNearestNeighbourTour()
{
	Reset();

	std::set<int> city_set;
	std::set<int>::iterator it;

	for ( int i = 0; i < matrix->size(); i++ )
	{
		city_set.insert( i );
	}

	int city = 0;	
	
	for ( int i = 1; i <= matrix->size(); i++ )
	{
		// Add city to tour
		cities.push_back( city );

		// Remove city from unique set
		it = city_set.find( city );
		city_set.erase( it );

		city = GetNearestNeighbour( city, city_set );		
	}
}

// Get the nearest node to this one
int Tour::GetNearestNeighbour( const int& c, std::set<int>& cset )
{
	int city = 0;

	// Get minimum distance node
	double min_dist = 99999999;
	std::set<int>::iterator cit;

	 for ( cit = cset.begin(); cit != cset.end(); cit++ )
	 {
		 int n = *cit;

		 if ( n == c ) continue;

		 double dist = Distance( c, n );
		
		 if ( dist < min_dist )
		 {
			city = n;
			min_dist = dist;
		 }
	 }
    
	return city;
}

// Set the cities
void Tour::SetCities( const std::vector<int>& v )
{
	cities = v;
}

// Set city
void Tour::SetCity( const int& index, const int& value )
{
	cities[ index ] = value;
}


// Copy / clone the tour cities to another vector
void Tour::Copy( std::vector<int>& v )
{
	int size = TourSize();
	v.reserve( size );
	v = cities;
}

// Reverse path between the selected end points
void Tour::ReverseCities( const int& c1, const int& c2 )
{
	int c1_n = c1 < c2 ? c1 : c2;
	int c2_n = c1 < c2 ? c2 : c1;

	if ( c1_n == 0 ) c1_n = 1;

	std::vector<int>::iterator it1, it2;

	it1 = cities.begin() + c1_n;
	it2 = cities.begin() + c2_n;
	
	reverse( it1, it2 ); 
}

// Rotate path cities given the beginning, middle and end points
void Tour::RotateCities( const int& c1, 
						 const int& c2,
						 const int& c3 )
{
	int c1_new = c1 == 0 ? 1 : c1;

	std::vector<int>::iterator it1 = cities.begin() + c1_new;
	std::vector<int>::iterator it2 = cities.begin() + c2;
	std::vector<int>::iterator it3 = cities.begin() + c3;

	rotate( it1, it2, it3 );
}