
// Two-OptDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Two-Opt.h"
#include "Two-OptDlg.h"
#include <fstream>
#include <string>
#include "afxdialogex.h"
#include "MakeString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

const int bottom_margin = 150;

const float coolrate = (float) 0.95;
const int iterations = 10000;
const int temp = 10000;
//const int temp = 0;

// CTwoOptDlg dialog


CTwoOptDlg::CTwoOptDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CTwoOptDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CTwoOptDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_STATIC_DIST, distText);
	DDX_Control(pDX, IDC_STATIC_ITER, iterText);
}

BEGIN_MESSAGE_MAP(CTwoOptDlg, CDialogEx)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RUN2, &CTwoOptDlg::OpenTestProblem)
	ON_BN_CLICKED(IDC_RUN, &CTwoOptDlg::RunAlgorithm)
END_MESSAGE_MAP()


// CTwoOptDlg message handlers

BOOL CTwoOptDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here
	Init( &alg );
	alg.Initialize( iterations, 
		            &mat, 
					(const float& ) temp, 
					(const float& ) coolrate );
	
	hasRun = false;

	// Creates a 8-point-Arial-font
	font.CreatePointFont(80, _T("Arial"));

	// With a member variable associated to the static control
	distText.SetFont(&font);
	iterText.SetFont(&font);
		
	GetDlgItem(IDC_STATIC_DIST)->SetFont(&font);
	GetDlgItem(IDC_STATIC_ITER)->SetFont(&font);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CTwoOptDlg::OnPaint()
{	
	// device context for painting
	CPaintDC dc(this);

	if (IsIconic())
	{		
		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);		
	}
	else
	{	
		if ( mat.size() < 1 ) return;

		CRect rect;
		GetClientRect(&rect);

		int rectx1 = rect.left + 20;
		int rectx2 = rect.right  - 150;
		int recty1 = rect.top + 20;
		int recty2 = rect.bottom - bottom_margin;

		CPen pen( PS_SOLID, 
				  1, 
				  RGB( 0, 0, 0 ) );

		dc.SelectStockObject(BLACK_BRUSH);		

		// Plot each node
		int tour_size = mat.size();
		
		for ( count = 0; count < tour_size; count++ )
		{
			xc1 = mat.GetX( count );
			yc1 = mat.GetY( count );

			xn1 = (float) ( xc1 - mat.GetMinX() ) / (float) ( mat.GetMaxX() - mat.GetMinX() );
			yn1 = (float) ( yc1 - mat.GetMinY() ) / (float) ( mat.GetMaxY() - mat.GetMinY() );

			xcoord1 = rectx1 + (int) (float) ( xn1 * abs( rectx1 - rectx2 ) );
			ycoord1 = recty2 - (int) (float) ( yn1 * abs( recty1 - recty2 ) );

			dc.Ellipse( xcoord1 - 2, ycoord1 - 2, xcoord1 + 2, ycoord1 + 2 );	

			if ( hasRun && count < tour_size - 1 )
			{
				cc1 = alg.GetTourCity( count );
				cc2 = alg.GetTourCity( count + 1 );

				xc1 = mat.GetX( cc1 );
				yc1 = mat.GetY( cc1 );
				xc2 = mat.GetX( cc2 );
				yc2 = mat.GetY( cc2 );

				xn1 = (float) ( xc1 - mat.GetMinX() ) / (float) ( mat.GetMaxX() - mat.GetMinX() );
				yn1 = (float) ( yc1 - mat.GetMinY() ) / (float) ( mat.GetMaxY() - mat.GetMinY() );
				xn2 = (float) ( xc2 - mat.GetMinX() ) / (float) ( mat.GetMaxX() - mat.GetMinX() );
				yn2 = (float) ( yc2 - mat.GetMinY() ) / (float) ( mat.GetMaxY() - mat.GetMinY() );

				xcoord1 = rectx1 + (int) (float) ( xn1 * abs( rectx1 - rectx2 ) );
				ycoord1 = recty2 - (int) (float) ( yn1 * abs( recty1 - recty2 ) );
				xcoord2 = rectx1 + (int) (float) ( xn2 * abs( rectx1 - rectx2 ) );
				ycoord2 = recty2 - (int) (float) ( yn2 * abs( recty1 - recty2 ) );

				dc.MoveTo( xcoord1, ycoord1 );
				dc.LineTo( xcoord2, ycoord2 );
			}
		}	

		// Draw final link
		if ( hasRun )
		{
			cc1 = alg.GetTourCity( tour_size - 1 );
			cc2 = alg.GetTourCity( 0 );

			xc1 = mat.GetX( cc1 );
			yc1 = mat.GetY( cc1 );
			xc2 = mat.GetX( cc2 );
			yc2 = mat.GetY( cc2 );

			xn1 = (float) ( xc1 - mat.GetMinX() ) / (float) ( mat.GetMaxX() - mat.GetMinX() );
			yn1 = (float) ( yc1 - mat.GetMinY() ) / (float) ( mat.GetMaxY() - mat.GetMinY() );
			xn2 = (float) ( xc2 - mat.GetMinX() ) / (float) ( mat.GetMaxX() - mat.GetMinX() );
			yn2 = (float) ( yc2 - mat.GetMinY() ) / (float) ( mat.GetMaxY() - mat.GetMinY() );

			xcoord1 = rectx1 + (int) (float) ( xn1 * abs( rectx1 - rectx2 ) );
			ycoord1 = recty2 - (int) (float) ( yn1 * abs( recty1 - recty2 ) );
			xcoord2 = rectx1 + (int) (float) ( xn2 * abs( rectx1 - rectx2 ) );
			ycoord2 = recty2 - (int) (float) ( yn2 * abs( recty1 - recty2 ) );

			dc.MoveTo( xcoord1, ycoord1 );
			dc.LineTo( xcoord2, ycoord2 );
		}

		CDialogEx::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CTwoOptDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}



// Open the selected test problem
void CTwoOptDlg::OpenTestProblem()
{
	CPaintDC dc(this);
	CRect rect;
	GetClientRect(&rect);

	// Open the file opening dialog
	CFileDialog fileDlg( TRUE, 
						 NULL, 
						 NULL, 
						 OFN_HIDEREADONLY | OFN_FILEMUSTEXIST, 
						 (LPCTSTR) ".tsp Files(*.tsp)|*.tsp||",
						 this );

	fileDlg.m_ofn.lpstrTitle = (LPCSTR)"Loading TSP Problem";

	INT_PTR result = fileDlg.DoModal();

	CString path = fileDlg.GetPathName();	
	
	if ( result == IDOK  )
	{
		// Reset matrix data contents		
		CT2CA pszConvertedAnsiString( path );
        std::string path_str( pszConvertedAnsiString );		
		mat.Initialize( path_str );		
		hasRun = false;

		(AfxGetMainWnd( ))->SetWindowText(mat.GetFileTitle().c_str());
	}

	// Re-draw the window	
	RedrawWindow( 0, 0, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE );	
}


void CTwoOptDlg::RunAlgorithm()
{
	alg.Run();
	hasRun = true;
	// Re-draw the window	
	RedrawWindow( 0, 0, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE );	
}


// Initialise obersers
void CTwoOptDlg::Init( TSPalgorithm* tsp )
{
	alg.AddObserver( this );  
}

    

// Tell the GUI to update display
void CTwoOptDlg::UpdateDisplay( const int& dist, const int& iter )
{
	hasRun = true;
	
	// Set the new distance text
	std::string s = make_string() << "Total distance: " << dist;
	distText.SetWindowText( s.c_str() );	

	// Set the iterationtext
	std::string i = make_string() << "Iteration: " << iter + 1;
	iterText.SetWindowText( i.c_str() );	
		
	// Re-draw the window	
	RedrawWindow( 0, 0, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE );	
}